/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import java.util.Collection;
import java.util.Collections;
import java.util.Optional;

import jakarta.validation.ConstraintValidator;
import jakarta.validation.ConstraintValidatorContext;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
public class UploadBinaryFileContentTypeValidator implements ConstraintValidator<BinaryFileContentTypeConstraint, UploadBinaryFileRequest> {

	private final BinaryFileProperties properties;

	@Override
	public boolean isValid(UploadBinaryFileRequest uploadRequest, ConstraintValidatorContext context) {
		var contentTypes = getConfiguredContentTypes(uploadRequest.getField());
		return contentTypes.isEmpty() || contentTypes.contains(uploadRequest.getContentType());
	}

	Collection<String> getConfiguredContentTypes(String field) {
		return Optional.ofNullable(properties.getContentTypes().get(field)).orElse(Collections.emptyList());
	}
}
