/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;

import de.ozgcloud.alfa.common.LinkedResourceProcessor;

class CollaborationLinkedResourceTest {

	@InjectMocks
	private LinkedResourceProcessor<Collaboration> processor;

	@Nested
	class TestEntityModelProcessing {
		private static final String REL_ZUSTAENDIGE_STELLE = "zustaendigeStelle";

		@Test
		void shouldHaveLinkToOrganisationsEinheit() {
			var entityModel = processor.process(EntityModel.of(OrganisationsEinheitCollaborationTestFactory.create()));

			assertThat(entityModel.getLink(REL_ZUSTAENDIGE_STELLE)).get().extracting(Link::getHref)
					.isEqualTo(OrganisationsEinheitController.PATH + "/" + OrganisationsEinheitTestFactory.ID);
		}

		@Test
		void shouldHaveLinkToFachstelle() {
			var entityModel = processor.process(EntityModel.of(FachstelleCollaborationTestFactory.create()));

			assertThat(entityModel.getLink(REL_ZUSTAENDIGE_STELLE)).get().extracting(Link::getHref)
					.isEqualTo(FachstelleController.PATH + "/" + FachstelleTestFactory.ID);
		}

	}
}
