/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import com.thedeanda.lorem.LoremIpsum;

class OrganisationsEinheitServiceTest {

	@InjectMocks
	private OrganisationsEinheitService service;

	@Mock
	private OrganisationsEinheitRemoteService remoteService;

	@Nested
	class TestGetById {

		@Test
		void shouldCallRemoteService() {
			service.getById(OrganisationsEinheitTestFactory.ID);

			verify(remoteService).getById(OrganisationsEinheitTestFactory.ID);
		}

		@Test
		void shouldReturnOrganisationsEinheit() {
			var organisationsEinheit = OrganisationsEinheitTestFactory.create();
			when(remoteService.getById(OrganisationsEinheitTestFactory.ID)).thenReturn(organisationsEinheit);

			var returnedOrganisationsEinheit = service.getById(OrganisationsEinheitTestFactory.ID);

			assertThat(returnedOrganisationsEinheit).isEqualTo(organisationsEinheit);
		}
	}

	@Nested
	class TestSearchOrganisationsEinheiten {

		private final String searchBy = LoremIpsum.getInstance().getWords(5);

		@Test
		void shouldCallRemoteService() {
			callService();

			verify(remoteService).search(searchBy);
		}

		@Test
		void shouldReturnOrganisationsEinheiten() {
			var organisationsEinheitHeader = OrganisationsEinheitHeaderTestFactory.create();
			when(remoteService.search(searchBy)).thenReturn(Stream.of(organisationsEinheitHeader));

			var organisationsEinheitHeaders = callService();

			assertThat(organisationsEinheitHeaders).containsExactly(organisationsEinheitHeader);
		}

		private Stream<OrganisationsEinheitHeader> callService() {
			return service.searchOrganisationsEinheiten(searchBy);
		}
	}

}
