/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.attacheditem;

import static org.assertj.core.api.Assertions.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandTestFactory;

class VorgangAttachedItemHistorieProcessorTest {

	@Spy
	private VorgangAttachedItemHistorieProcessor processor;

	@Nested
	class TestProcess {

		@ParameterizedTest
		@EnumSource(names = { "DELETE_ATTACHED_ITEM" }, mode = Mode.EXCLUDE)
		void shouldKeepCommand(CommandOrder order) {
			var command = CommandTestFactory.createBuilder().order(order.name()).build();

			var commands = processor.process(Stream.of(command));

			assertThat(commands).containsExactly(command);
		}

		@ParameterizedTest
		@EnumSource(names = { "DELETE_ATTACHED_ITEM" }, mode = Mode.INCLUDE)
		void shouldRemoveCommand(CommandOrder order) {
			var command = CommandTestFactory.createBuilder().order(order.name()).build();

			var commands = processor.process(Stream.of(command));

			assertThat(commands).isEmpty();
		}
	}
}
