/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static org.assertj.core.api.Assertions.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.springframework.hateoas.IanaLinkRelations;

import de.ozgcloud.alfa.common.file.OzgFileTestFactory;

class BinaryFileModelAssemblerTest {

	@InjectMocks
	private BinaryFileModelAssembler modelAssembler;

	@Nested
	class TestLinksOnModel {

		@Test
		void shouldHaveSelfLink() {
			var link = modelAssembler.toModel(OzgFileTestFactory.create()).getLink(IanaLinkRelations.SELF_VALUE);

			assertThat(link).isPresent();
			assertThat(link.get().getHref()).isEqualTo(BinaryFileController.PATH + "/" + OzgFileTestFactory.ID);
		}

		@Test
		void shouldHaveDownloadLink() {
			var link = modelAssembler.toModel(OzgFileTestFactory.create()).getLink(BinaryFileModelAssembler.REL_DOWNLOAD);

			assertThat(link).isPresent();
			assertThat(link.get().getHref()).isEqualTo(BinaryFileController.PATH + "/" + OzgFileTestFactory.ID);
		}
	}

	@Nested
	class TestLinksOnCollectionModel {

		@Test
		void shouldHaveSelfLink() {
			var link = modelAssembler.toCollectionModel(Stream.of(OzgFileTestFactory.create())).getLink(IanaLinkRelations.SELF_VALUE);

			assertThat(link).isPresent();
			assertThat(link.get().getHref()).isEqualTo(BinaryFileController.PATH);
		}
	}
}