/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.common.LinkedResourceProcessor;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;

class CommandLinkedResourceTest {

	@InjectMocks
	private LinkedResourceProcessor<Command> processor;
	@Mock
	private UserManagerUrlProvider userManagerUrlProvider;

	@Nested
	class TestEntityModelProcessing {
		private static final String CREATED_BY = "createdBy";
		private static final String ROOT_URL = LoremIpsum.getInstance().getUrl();
		private static final String USER_PROFILES_API_PATH = "/api/user-profiles/";

		@Test
		void shouldHaveCreatedByLink() {
			when(userManagerUrlProvider.getUserProfileTemplate()).thenReturn(ROOT_URL + USER_PROFILES_API_PATH + "%s");

			var model = processor.process(EntityModel.of(CommandTestFactory.create()));

			assertThat(model.getLink(CREATED_BY)).isPresent().get().extracting(Link::getHref)
					.isEqualTo(ROOT_URL + USER_PROFILES_API_PATH + UserProfileTestFactory.ID);
		}

		@Test
		void shouldNotHaveCreatedByLinkForSystemUser() {
			var systemUser = UserId.from("system-" + LoremIpsum.getInstance().getWords(1));
			var commandBySystemUser = CommandTestFactory.createBuilder()
					.createdBy(systemUser)
					.build();

			var model = processor.process(EntityModel.of(commandBySystemUser));

			assertThat(model.getLink(CREATED_BY)).isNotPresent();
		}

		@Test
		void shouldNotHaveCreatedByLinkOnNullValue() {
			var model = processor.process(EntityModel.of(CommandTestFactory.createBuilder().createdBy(null).build()));

			assertThat(model.getLink(CREATED_BY)).isNotPresent();
		}
	}
}
